var Notes = Notes || {};

Notes.controller = (function ($, dataContext) {

    var notesListPageId = "notes-list-page";
    var notesListSelector = "#notes-list-content";
    var appStorageKey = "Notes.NotesList";
    var noteEditorPageId = "note-editor-page";
    var noteTimeEditorSel = "[name=note-time-editor]",
        noteLengthEditorSel = "[name=note-length-editor]",
		noteReasonEditorSel = "[name=note-reason-editor]";
		noteFeelingEditorSel = "[name=note-feeling-editor]";
		noteIntensityEditorSel = "[name=note-intensity-editor]";
		noteHelperEditorSel = "[name=note-helper-editor]";
    var saveNoteButtonSel = "#save-note-button";
    var invalidNoteDlgSel = "#invalid-note-dialog";
    var defaultDlgTrsn = { transition: "slideup" };
    var confirmDeleteNoteDlgSel = "#confirm-delete-note-dialog";
    var deleteNoteButtonSel = "#delete-note-button",
        deleteNoteContentPlaceholderSel = "#delete-note-content-placeholder",
        okToDeleteNoteButtonSel = "#ok-to-delete-note-button";
    var syncNotesWithServerBtnSel = "#sync-notes-button";
    var defaultLoadingMsg,
        syncingMsg = "Syncing Notes";
    var appServerUrl = "notes.php";

    var queryStringToObject = function (queryString) {

        var queryStringObj = {};
        var e;
        var a = /\+/g;  // Replace + symbol with a space
        var r = /([^&;=]+)=?([^&;]*)/g;
        var d = function (s) { return decodeURIComponent(s.replace(a, " ")); };

        e = r.exec(queryString);
        while (e) {
            queryStringObj[d(e[1])] = d(e[2]);
            e = r.exec(queryString);

        }

        return queryStringObj;
    };

    var returnToNotesListPage = function () {

        $.mobile.changePage("#" + notesListPageId,
        { transition: "slide", reverse: true });
    };

    var resetCurrentNote = function () {
        currentNote = null;
    };

    var renderNotesList = function () {

        var notesList = dataContext.getNotesList();
        var view = $(notesListSelector);

        view.empty();

        if (notesList.length === 0) {

            $(noNotesCachedMsg).appendTo(view);
        } else {

            var liArray = [],
            notesCount,
            note,
            dateGroup,
            noteDate,
            i,
            ul,
            liHtml,
            liIcon,
            lastUpdateDate,
            lastUploadDate;

            notesCount = notesList.length,
        ul = $("<ul id=\"notes-list\" data-role=\"listview\" data-divider-theme=\"c\"></ul>").appendTo(view);

            for (i = 0; i < notesCount; i += 1) {

                note = notesList[i];

                noteDate = (new Date(note.dateCreated)).toDateString();

                // We will not render notes that are marked for deletion.
                if (!note.deleteAfterUpload) {

                    if (dateGroup !== noteDate) {
                        liArray.push("<li data-role=\"list-divider\" data-theme=\"e\">" + noteDate + "</li>");
                        dateGroup = noteDate;
                    }

                    lastUpdateDate = new Date(note.lastUpdateDate).getTime();
                    lastUploadDate = new Date(note.lastUploadDate).getTime();

                    if (lastUpdateDate < lastUploadDate) {
                        liIcon = "<img src=\"images/loop_green.png\" alt=\"Uploaded\" class=\"ui-li-icon\">";
                    } else {
                        liIcon = "<img src=\"images/loop_red.png\" alt=\"Uploaded\" class=\"ui-li-icon\">"; ;
                    }

                    liHtml = "<li>"
                + "<a data-url=\"logbook.html#note-editor-page?noteId=" + note.id + "\" href=\"logbook.html#note-editor-page?noteId=" + note.id + "\">"
                + liIcon
                + "<div  class=\"list-item-time\"><table><tr><td width=80px><b><small>Startede Kl:</small></b></td><td><small> " + note.time + "</small></td></tr></table></div>"
                + "<div class=\"list-item-length\"><table><tr><td width=80px><small><b>Varighed:</b></small> </td><td><small>" + note.length + " minutter</small></td></tr></table></div>"
				+ "<div class=\"list-item-intensity\"><table><tr><td width=80px><small><b>Styrke:</b></small> </td><td><small>" + note.intensity + "</small></td></tr></table></div>"
                + "</a>"
                + "</li>"

                    liArray.push(liHtml);
                }

            }

            var listItems = liArray.join("");
            $(listItems).appendTo(ul);


            ul.listview();
        }
    };

    var renderSelectedNote = function (data) {

        var u = $.mobile.path.parseUrl(data.options.fromPage.context.URL);
        var re = "^#" + noteEditorPageId;

        if (u.hash.search(re) !== -1) {

            var queryStringObj = queryStringToObject(data.options.queryString);

            var timeEditor = $(noteTimeEditorSel);
            var lengthEditor = $(noteLengthEditorSel);
			var reasonEditor = $(noteReasonEditorSel);
			var feelingEditor = $(noteFeelingEditorSel);
			var intensityEditor = $(noteIntensityEditorSel);
			var helperEditor = $(noteHelperEditorSel);

            var noteId = queryStringObj["noteId"];

            if (typeof noteId !== "undefined") {

                // We're editing an existing note. Load the note’s properties in the editor. 
                var notesList = dataContext.getNotesList();
                var notesCount = notesList.length;
                var note;

                for (var i = 0; i < notesCount; i++) {

                    note = notesList[i];

                    if (noteId === note.id) {
                        currentNote = note;
                        timeEditor.val(currentNote.time);
                        lengthEditor.val(currentNote.length);
						reasonEditor.val(currentNote.reason);
						feelingEditor.val(currentNote.feeling);
						intensityEditor.val(currentNote.intensity);
						helperEditor.val(currentNote.helper);
                    }
                }

            } else {
                // We're creating a note. Reset the fields.
                timeEditor.val("");
                lengthEditor.val("");
				reasonEditor.val("");
				feelingEditor.val("");
				intensityEditor.val("");
				helperEditor.val("");
            }

            timeEditor.focus();
        }
    };

    var onPageBeforeChange = function (event, data) {

        if (typeof data.toPage === "string") {

            var url = $.mobile.path.parseUrl(data.toPage);

            if ($.mobile.path.isEmbeddedPage(url)) {

                data.options.queryString = $.mobile.path.parseUrl(url.hash.replace(/^#/, "")).search.replace("?", "");
            }
        }
    };

    var onPageChange = function (event, data) {

        var toPageId = data.toPage.attr("id");
        var fromPageId = null;

        if (data.options.fromPage) {
            fromPageId = data.options.fromPage.attr("id");
        }

        switch (toPageId) {

            case notesListPageId:
                resetCurrentNote(); // <-- Reset reference to the note being edited.
                renderNotesList();
                break;

            case noteEditorPageId:

                if (fromPageId === notesListPageId) {
                    renderSelectedNote(data);
                }
                break;
        }
    };

    var onSaveNoteButtonTapped = function () {

        // Validate note.
        var timeEditor = $(noteTimeEditorSel);
        var lengthEditor = $(noteLengthEditorSel);
		var reasonEditor = $(noteReasonEditorSel);
		var feelingEditor = $(noteFeelingEditorSel);
		var intensityEditor = $(noteIntensityEditorSel);
		var helperEditor = $(noteHelperEditorSel);
        var tempNote = dataContext.createBlankNote();


        tempNote.time = timeEditor.val();
        tempNote.length = lengthEditor.val();
		tempNote.reason = reasonEditor.val();
		tempNote.feeling = feelingEditor.val();
		tempNote.intensity = intensityEditor.val();
		tempNote.helper = helperEditor.val();
        tempNote.lastUpdateDate = new Date();

        if (tempNote.isValid()) {

            if (null !== currentNote) {

                if (currentNote.time !== tempNote.time || currentNote.length !== tempNote.length || currentNote.reason !== tempNote.reason || currentNote.feeling !== tempNote.feeling || currentNote.intensity !== tempNote.intensity || currentNote.helper !== tempNote.helper) {

                    currentNote.time = tempNote.time;
                    currentNote.length = tempNote.length;
					currentNote.reason = tempNote.reason;
					currentNote.feeling = tempNote.feeling;
					currentNote.intensity = tempNote.intensity;
					currentNote.helper = tempNote.helper;
                    currentNote.lastUpdateDate = tempNote.lastUpdateDate; // Update last upload date.
                }

            } else {

                currentNote = tempNote;
            }

            $.mobile.showPageLoadingMsg();

            dataContext.saveNote(currentNote);

            $.mobile.hidePageLoadingMsg();

            returnToNotesListPage();

        } else {
            $.mobile.changePage(invalidNoteDlgSel, defaultDlgTrsn);
        }
    };

    var onDeleteNoteButtonTapped = function () {

        if (currentNote) {
            // Render selected note in confirmation dlg.
            // Deletion will be handled elsewhere, after user confirms it's ok to delete.

            var noteContentPlaceholder = $(deleteNoteContentPlaceholderSel);

            noteContentPlaceholder.empty();
            $("<h3>" + currentNote.time + "</h3><p>" + currentNote.length + "</p>").appendTo(noteContentPlaceholder);

            $.mobile.changePage(confirmDeleteNoteDlgSel, defaultDlgTrsn);
        }
    };

    var onOKToDeleteNoteButtonTapped = function () {

        $.mobile.showPageLoadingMsg()

        //dataContext.deleteNote(currentNote);
        currentNote.deleteAfterUpload = true;
        dataContext.saveNote(currentNote);

        $.mobile.hidePageLoadingMsg();

        returnToNotesListPage();
    };

    var onSyncNotesWithServerButtonTapped = function () {

        $.mobile.loadingMessage = syncingMsg;
        $.mobile.showPageLoadingMsg();
        dataContext.startNotesSync(onNotesSyncCompleted);
    };

    var onNotesSyncCompleted = function (syncResult) {

        renderNotesList();
        $.mobile.hidePageLoadingMsg();
        $.mobile.loadingMessage = defaultLoadingMsg;
    };

    var init = function () {

        dataContext.init({
            storageKey: appStorageKey,
            serverUrl: appServerUrl
        });

        var d = $(document);
        d.bind("pagebeforechange", onPageBeforeChange);
        d.bind("pagechange", onPageChange);
        d.delegate(saveNoteButtonSel, "tap", onSaveNoteButtonTapped);
        d.delegate(deleteNoteButtonSel, "tap", onDeleteNoteButtonTapped);
        d.delegate(okToDeleteNoteButtonSel, "tap", onOKToDeleteNoteButtonTapped);
        d.delegate(syncNotesWithServerBtnSel, "tap", onSyncNotesWithServerButtonTapped);

        defaultLoadingMsg = $.mobile.loadingMessage;
    };



    var public = {
        init: init
    };

    return public;

})(jQuery, Notes.dataContext);

$(document).bind("mobileinit", function () {
    //Notes.testHelper.createDummyNotes("Notes.NotesList");
    Notes.controller.init();
});


