var Notes = Notes || {};

Notes.dataContext = (function () {

    var notesList = [],
        notesListStorageKey,
        serverUrl;

    var getRandomInt = function (min, max) {
        return Math.floor(Math.random() * (max - min + 1)) + min;
    };

    var getNotesList = function () {
        return notesList;
    };

    var loadNotesFromLocalStorage = function () {

        var storedNotes = $.jStorage.get(notesListStorageKey);

        if (storedNotes !== null) {
            notesList = storedNotes;
        } else {
            notesList = [];
        }
    };

    var saveNotesToLocalStorage = function () {
        $.jStorage.set(notesListStorageKey, notesList);
    };

    var createBlankNote = function () {

        var dateCreated = new Date();
        var id = dateCreated.getTime().toString() + (getRandomInt(0, 100)).toString();
        var noteModel = new Notes.NoteModel({
            id: id,
            dateCreated: dateCreated,
            time: "",
			length: "",
			reason: "",
            feeling: "",
			intensity: "",
			helper: ""
        });

        return noteModel;
    };

    var saveNote = function (noteModel) {

        var found = false;
        var i;

        for (i = 0; i < notesList.length; i += 1) {
            if (notesList[i].id === noteModel.id) {
                notesList[i] = noteModel;
                found = true;
                i = notesList.length;
            }
        }

        if (!found) {
            notesList.splice(0, 0, noteModel);
        }

        saveNotesToLocalStorage();
    };

    var deleteNote = function (noteModel) {

        var i;
        for (i = 0; i < notesList.length; i += 1) {
            if (notesList[i].id === noteModel.id) {
                notesList.splice(i, 1);
                i = notesList.length;
            }
        }

        saveNotesToLocalStorage();
    };

    var startNotesSync = function (callback) {

        $.ajax({
            url: serverUrl,
            type: "post",
            data: "notes=" + JSON.stringify(notesList),
            success: function (data, textStatus, jqXHR) {

                var syncResult = new Notes.AjaxResult({
                    success: true,
                    errorThrown: null
                });

                endNotesSync(syncResult, data, callback);
            },
            error: function (jqXHR, textStatus, errorThrown) {


                var syncResult = new Notes.AjaxResult({
                    success: false,
                    errorThrown: errorThrown
                });

                endNotesSync(syncResult, null, callback);
            }
        });
    };

    var endNotesSync = function (result, data, callback) {

        var note,
        syncResult,
        syncResults,
        idsOfNotesToDelete = [],
        notesListLength,
        i, j,
        okToContinue = true;

        if (result.success && data) {

            notesListLength = notesList.length;
            syncResults = data;

            // CRUD based on server results.
            for (i = 0; i < notesListLength; i += 1) {
                for (j = 0; j < syncResults.length; j += 1) {

                    note = notesList[i];
                    syncResult = syncResults[j];

                    if (syncResult.isNewNote) {

                        // Add any new notes. 
                        notesList.splice(0, 0, syncResult, note);
                    } else {

                        // Update notes that made it to the server.
                        if (syncResult.note.id === note.id) {

                            note.time = syncResult.note.time;
                            note.length = syncResult.note.length;
							note.reason = syncResult.note.reason;
							note.feeling = syncResult.note.feeling;
							note.intensity = syncResult.note.intensity;
							note.helper = syncResult.note.helper;

                            note.lastUploadDate = new Date();

                            // Find out if we need to delete this note.
                            if (note.deleteAfterUpload) {
                                idsOfNotesToDelete.push(syncResult.note.id);
                            }
                        }
                    }
                }
            }

            // Delete the notes that were marked for deletion and were successfully uploaded.
            for (i = 0; i < idsOfNotesToDelete.length; i += 1) {

                notesListLength = notesList.length;
                okToContinue = true;

                for (j = 0; j < notesListLength && okToContinue; j += 1) {

                    if (idsOfNotesToDelete[i] === notesList[j].id) {
                        notesList.splice(j, 1);
                        okToContinue = false;
                    }
                }
            }

            saveNotesToLocalStorage();
        }
        callback();
    };

    var init = function (config) {
        notesListStorageKey = config.storageKey;
        serverUrl = config.serverUrl;
        loadNotesFromLocalStorage();
    };

    var public = {
        init: init,
        getNotesList: getNotesList,
        createBlankNote: createBlankNote,
        saveNote: saveNote,
        deleteNote: deleteNote,
        startNotesSync: startNotesSync
    };

    return public;

})();

